-----------------------------------------------------------------------------
--
-- data
--   \damageDealt
--     \<damageDealerName>
--       -count, amount, absorbAmount, blockAmount, resistAmount
--       \attackDetails
--         \<attackName>
--            -count, amount, absorbAmount, blockAmount, resistAmount
--			  \<attackType> -- one of Hit, Critical Hit, Tick, Miss, Dodge, Resist, etc.
--              -count, amount, absorbAmount, blockAmount, resistAmount
--   \damageTaken
--     \<damageTakerName>
--       -count, amount, absorbAmount, blockAmount, resistAmount
--       \attackDetails
--         \<attackName>
--            -count, amount, absorbAmount, blockAmount, resistAmount
--			  \<attackType>
--              -count, amount, absorbAmount, blockAmount, resistAmount
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentDamage")
L:RegisterTranslations("enUS", function() return {
	["Damage Dealers"] = true,
	["Damage Takers"] = true,
	["Damage Dealt"] = true,
	["DPS"] = true,
} end)

-----------------------------------------------------------------------------

local AssessmentDamage = Assessment:NewModule("Damage", "AceEvent-2.0", "Parser-3.0")
AssessmentDamage.db = Assessment:AcquireDBNamespace("Damage")
Assessment:RegisterDefaults("Damage", "profile", {
	enable = true
})

-----------------------------------------------------------------------------

Assessment.TagData.damagedealt =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local damageDealt = fightData.damageDealt
				if not damageDealt then return 0 end
				
				if not damageDealt[data.key] then return 0 end
				return damageDealt[data.key].amount
			 end,
	data = "damage",
	guaranteeResult = true,
	numeric = true,
}

Assessment.TagData.damagetaken =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local damageTaken = fightData.damageTaken
				if not damageTaken then return 0 end
				
				if not damageTaken[data.key] then return 0 end
				return damageTaken[data.key].amount
			 end,
	data = "damage",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.DamageDealers =
{
	method = function(fightData)
				if not fightData.damageDealt then return function() return nil end, nil, nil end
				return pairs(fightData.damageDealt)
			 end,
	data = "damage",
	description = L["Damage Dealers"],
}

Assessment.ListType.DamageTakers =
{
	method = function(fightData)
				if not fightData.damageTaken then return function() return nil end, nil, nil end
				return pairs(fightData.damageTaken)
			 end,
	data = "damage",
	description = L["Damage Takers"],
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.DamageDealt =
{
	titleTag		= "Damage Dealt: [totalBarValue:short]",
	listType		= "GroupMembers", 
	barValueTag		= "damageDealt", 
	filterType		= "NONZERO",
	fightIndex		= 0,
	barValueTextTag	= "[barValue:short]",
	description		= L["Damage Dealt"],
}

Assessment.DefaultDisplaySets.DPS =
{
	titleTag		= "DPS: [combattime:format(\"%.1f\")]s",
	listType		= "GroupMembers", 
	barValueTag		= "damageDealt / maximum(combatTime, 1)", 
	filterType		= "NONZERO",
	fightIndex		= 1,
	barValueTextTag	= "[barValue:format(\"%.1f\")]",
	description		= L["DPS"],
}

-----------------------------------------------------------------------------

Assessment.DefaultTags.DPS =
{
	tag			= "[(damageDealt / maximum(combatTime, 1)):format(\"%.1f\")]",
	fightIndex	= 1,
	description	= L["DPS"],
}

-----------------------------------------------------------------------------

function AssessmentDamage:OnInitialize()
end

function AssessmentDamage:OnEnable()
	self:RegisterParserEvent({ eventType = "Damage" }, "OnParserDamageEvent")
end

function AssessmentDamage:OnDisable()
end

-----------------------------------------------------------------------------

local function AccumulateData(data, eventData)
	data.count = data.count + 1
	data.amount = data.amount + eventData.amount
	if eventData.absorbAmount ~= 0 then
		data.absorbAmount = (data.absorbAmount or 0) + eventData.absorbAmount
	end
	if eventData.blockAmount ~= 0 then
		data.blockAmount = (data.blockAmount or 0) + eventData.blockAmount
	end
	if eventData.resistAmount ~= 0 then
		data.resistAmount = (data.resistAmount or 0) + eventData.resistAmount
	end
end

-----------------------------------------------------------------------------

function AssessmentDamage:NewDataTable()
	local data = self:NewTable()
	data.count = 0
	data.amount = 0
	return data
end

-----------------------------------------------------------------------------

function AssessmentDamage:UpdateDamageData(dataTable, member, eventData, attackType)
	local data = dataTable[member]
	if not data then
		data = self:NewDataTable()
		data.attackDetails = self:NewTable()
		dataTable[member] = data
	end
	AccumulateData(data, eventData)
	
	local attackDetailsParent = data.attackDetails
	local attackDetails = attackDetailsParent[eventData.abilityName]
	if not attackDetails then
		attackDetails = self:NewDataTable()
		attackDetailsParent[eventData.abilityName] = attackDetails
	end
	AccumulateData(attackDetails, eventData)
	
	local attackTypeDetails = attackDetails[attackType]
	if not attackTypeDetails then
		attackTypeDetails = self:NewDataTable()
		attackDetails[attackType] = attackTypeDetails
	end
	AccumulateData(attackTypeDetails, eventData)
end

-----------------------------------------------------------------------------

function AssessmentDamage:OnParserDamageEvent(eventData)
	local attackType = "Hit"
	if eventData.isDoT then
		attackType = "Tick"
	elseif eventData.isCrit then
		attackType = "Critical Hit"
	end
	
	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.damage = GetTime()
		local damageDealt = data.damageDealt
		if not damageDealt then
			damageDealt = self:NewTable()
			data.damageDealt = damageDealt
		end

		local damageTaken = data.damageTaken
		if not damageTaken then
			damageTaken = self:NewTable()
			data.damageTaken = damageTaken
		end

		self:UpdateDamageData(damageDealt, eventData.sourceName, eventData, attackType)
		self:UpdateDamageData(damageTaken, eventData.recipientName, eventData, attackType)
	end
end

-----------------------------------------------------------------------------
